#!/usr/bin/env python

import os
import sys
import subprocess
import datetime
from optparse import OptionParser, OptionValueError
import xml.etree.ElementTree as ET
import shutil
import shlex
import numpy as np
from scipy.interpolate import interp1d
import matplotlib.pyplot as plt

p = OptionParser(usage="""usage: %prog [options] <index_file> <output_dir>
Extracts flow-rate information from <index_file> and creates png files in <output_dir>

For example:
%prog index.xml picture_directory
""")
p.add_option("-v", action="store_true",        dest="verbose",  help="Verbose")
p.add_option("--mines", action="store", dest="mine_names", default="Ashton,Austar,Abel,BengMtPlsnt,BHPMtArth,BloomMait,BulgaOC,BulgaUG,Bylong,Dartbrook,Drayton,DraytonSouth,histDTI,HowickWest,HVO,Idemitsu,Integra,LC_Mann_My,Liddell,Mandalong,Mangoola_OC,Mben,MtOwGlend,MtPenny,MTW,NewstanAwaba,Raven_OC,Raven_UG,RixCreek,SpurHill,Tasman,UlanUG,UlanSO,Wallarah2,WamboU,WestWallsend,Wilp", help="Only include the named mines.  Default=%default")

(opts, args) = p.parse_args()

if len(args) != 2:
    sys.stderr.write("You must supply two arguments to this program\n");
    sys.exit(1)
(index_file, output_dir) = args

index_dir = os.path.dirname(os.path.abspath(index_file))
if opts.verbose: print "Root directory is", index_dir


try:
    sys.stdout.write("Parsing " + index_file + "\n")
    index_tree = ET.parse(index_file)
except:
    sys.stderr.write("Cannot parse " + index_file + "\n")
    sys.exit(2)


index_root = index_tree.getroot()

if not os.path.exists(output_dir):
    if opts.verbose: print "Creating model directory", output_dir
    os.makedirs(output_dir)

restricted_mines = set(opts.mine_names.split(","))

all_mines = None
for prd in index_root.findall("processed_raw_data"):
    if prd.get("name") == "mines":
        all_mines = prd
        break
if not all_mines:
    sys.stderr.write("Cannot find mines in " + index_file + "\n")

ba_name = {}
ba_name["Austar"] = "Austar UG"
ba_name["Abel"] = "Abel UG"
ba_name["BengMtPlsnt"] = "Bengalla and Mount Pleasant"
ba_name["BHPMtArth"] = "Mount Arthur"
ba_name["BloomMait"] = "Bloomfield"
ba_name["BulgaOC"] = "Bulga OC"
ba_name["BulgaUG"] = "Bulga UG"
ba_name["Dartbrook"] = "Dartbrook UG"
ba_name["Drayton"] = "Drayton OC"
ba_name["DraytonSouth"] = "Drayton South OC"
ba_name["histDTI"] = "Cumnock UG"
ba_name["HowickWest"] = "Hunter Valley Operations Howick West"
ba_name["HVO"] = "Hunter Valley Operations OC"
ba_name["Idemitsu"] = "Muswellbrook"
ba_name["LC_Mann_My"] = "Chain Valley UG and Mannering UG and Myuna UG"
ba_name["Liddell"] = "Liddell OC"
ba_name["Mandalong"] = "Mandalong UG"
ba_name["Mangoola_OC"] = "Mangoola OC"
ba_name["Mben"] = "Moolarben"
ba_name["MtOwGlend"] = "Mount Owen OC and Glendell OC and Ravensworth East OC"
ba_name["MtPenny"] = "Mount Penny CANCELLED"
ba_name["MTW"] = "Mount Thorley-Warkworth OC"
ba_name["NewstanAwaba"] = "Newstan UG and Awaba UG"
ba_name["Raven_OC"] = "Ravensworth Complex OC"
ba_name["Raven_UG"] = "Ravensworth UG"
ba_name["RixCreek"] = "Rixs Creek OC"
ba_name["SpurHill"] = "Spur Hill NOTMODELLED"
ba_name["Tasman"] = "Tasman UG"
ba_name["UlanUG"] = "Ulan West UG and Ulan 3 UG"
ba_name["UlanSO"] = "Ulan OC"
ba_name["Wallarah2"] = "Wallarah 2 UG"
ba_name["WamboU"] = "Wambo"
ba_name["WestWallsend"] = "West Wallsend"
ba_name["Wilp"] = "Wilpinjong OC"

for mine in all_mines.findall("mine"):
    mine_name = mine.get("name")
    basedir = os.path.join(index_dir, mine.get("basedir"))
    if not mine_name in restricted_mines:
        continue
    ban = mine_name
    if mine_name in ba_name: ban = ba_name[mine_name]
    if opts.verbose: print mine_name, ban
    some_acrd = False
    rates_bl = [[t, 0] for t in range(1980, 2061)]
    rates_acrd = [[t, 0] for t in range(1980, 2061)]
    for extr in mine.find("extractions").findall("extraction"):
        acrd = extr.get("acrd").strip()
        flow_fn = extr.get("flow_rates")
        f = open(os.path.join(basedir, flow_fn), 'r')
        rates = []
        for line in f:
            if not line.strip():
                continue
            line = line.strip()
            if line.startswith("flow"):
                rates.append(map(float, line.split("=")[1].split(",")))
        if rates[0][0] > 1980:
            rates = [[1980, 0]] + rates
        if rates[-1][0] < 2060:
            rates.append([2060, 0])
        f = interp1d([d[0] for d in rates], [d[1] for d in rates])
        if str.lower(acrd) == "false":
            rates_bl = [[tr[0], tr[1] + f(tr[0])] for tr in rates_bl]
        else:
            some_acrd = True
            rates_acrd = [[tr[0], tr[1] + f(tr[0])] for tr in rates_acrd]
    rates_crdp = [[rates_acrd[i][0], rates_acrd[i][1] + rates_bl[i][1]] for i in range(len(rates_acrd))]
    plt.figure()
    plt.plot([d[0] for d in rates_bl], [d[1] for d in rates_bl], 'b-', label='baseline', linewidth=5)
    if some_acrd:
        plt.plot([d[0] for d in rates_crdp], [d[1] for d in rates_crdp], 'r-', label='CRDP', linewidth=2)
    plt.legend(loc='upper right')
    plt.xlabel("Year")
    plt.ylabel("Rate (ML/year)")
    plt.title(ban)
    plt.savefig(os.path.join(output_dir, ban.replace(" ", "_") + ".png"))


sys.exit(0)
